// hpstat: High-performance statistics implementations
// Copyright © 2023  Lee Yingtong Li (RunasSudo)
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
// 
// You should have received a copy of the GNU Affero General Public License
// along with this program.  If not, see <https://www.gnu.org/licenses/>.

pub struct BisectionRootFinder {
	bound_lower: f64,
	bound_upper: f64,
	value_lower: f64,
	value_upper: f64
}

impl BisectionRootFinder {
	pub fn new(bound_lower: f64, bound_upper: f64, value_lower: f64, value_upper: f64,) -> BisectionRootFinder {
		return BisectionRootFinder {
			bound_lower: bound_lower,
			bound_upper: bound_upper,
			value_lower: value_lower,
			value_upper: value_upper
		}
	}
	
	pub fn update(&mut self, guess: f64, value: f64) {
		if value > 0.0 {
			if self.value_lower > 0.0 || self.value_upper < 0.0 {
				self.bound_lower = guess;
				self.value_lower = value;
			} else {
				self.bound_upper = guess;
				self.value_upper = value;
			}
		} else {
			if self.value_lower < 0.0 || self.value_upper > 0.0 {
				self.bound_lower = guess;
				self.value_lower = value;
			} else {
				self.bound_upper = guess;
				self.value_upper = value;
			}
		}
	}
	
	pub fn next_guess(&self) -> f64 {
		return (self.bound_lower + self.bound_upper) / 2.0;
	}
	
	pub fn precision(&self) -> f64 {
		return self.bound_upper - self.bound_lower;
	}
}
